import './FwAffixTable.less'
import draggable from 'vuedraggable'
export default {
  name: 'FwAffixTable',
  props: {
    stickyTop: {
      type: Number,
      default: 64
    },
    selector: {
      type: String,
      default: '#scrollContainer'
    },
    setConfig: {
      type: Boolean,
      default: false// 默认false不开启,防止改造不彻底引起bug
    }
  },
  watch: {
    '$attrs.columns': {
      handler (val) {
        this.preprocessSetConfig()
      },
      immediate: true
    }
  },
  data () {
    return {
      scrollContainer: null,
      scrollTop: 0,
      $head: null,
      $body: null,
      ticking: false,
      localSetConfig: { columns: [] }, //
      checkboxAllChecked: false, // 全选默认不勾选
      indeterminate: true, // true 半选状态
      filterDropdownVisible: false
    }
  },
  activated () {
    // 如果页面用户keep-alive，列表需要定位到对应项
    if (this.scrollTop && this.scrollContainer) {
      this.scrollContainer.scrollTop = this.scrollTop
      this.scrollContainer.addEventListener('scroll', this.onScroll)
    }
  },
  deactivated () {
    this.scrollContainer.removeEventListener('scroll', this.onScroll)
  },
  beforeDestroy () {
    this.scrollContainer.removeEventListener('scroll', this.onScroll)
  },
  mounted () {
    this.scrollContainer = document.querySelector(this.selector)
    this.$body = this.$el && this.$el.querySelector('.ant-table-content')
    this.$head = this.$el && this.$el.querySelectorAll('.ant-table-thead')
    this.setFixedStyle()
    this.scrollContainer.addEventListener('scroll', this.onScroll)
  },
  methods: {
    setFixedStyle () {
      this.$fixedTable = this.$el.querySelectorAll('.ant-table-body .ant-table-thead')
      for (let i = 0; i < this.$fixedTable.length; i++) {
        const item = this.$fixedTable[i]
        item.style.background = 'url(http://about:blank)'
        item.style.backgroundAttachment = 'fixed'
      }
    },
    onScroll () {
      this.scrollTop = this.scrollContainer.scrollTop
      if (!this.ticking) {
        window.requestAnimationFrame(() => {
          const bodyTop = this.$body.getBoundingClientRect().top
          for (let i = 0; i < this.$head.length; i++) {
            const item = this.$head[i]
            if (bodyTop <= this.stickyTop) {
              item.style.transform = `translate3d(0, ${this.stickyTop - bodyTop}px, 0)`
            } else {
              item.style.transform = `translate3d(0, 0, 0)`
            }
            item.style.zIndex = 1
            item.style.position = 'relative'
          }
          this.ticking = false
        })
        this.ticking = true
      }
    },
    /**
     * 预处理设置配置信息
     */
    preprocessSetConfig () {
      if (this.setConfig) {
        this.localSetConfig = {
          columns: [].concat(this.$attrs.columns || [])
        }
      }
      // 兼容处理数据 列头没有checked、disabled属性
      const columns = this.localSetConfig.columns
      if (Array.isArray(columns) && columns.length > 0) {
        columns.forEach((item, index) => {
          // 默认勾选
          if (!item.hasOwnProperty('checked')) {
            this.$set(item, 'checked', true)
          }
          // 默认不屏蔽
          if (!item.hasOwnProperty('disabled')) {
            this.$set(item, 'disabled', false)
          }
          // 必须有dataIndex属性
          if (!item.hasOwnProperty('dataIndex')) {
            this.$set(item, 'dataIndex', `${index}-${parseInt(Math.random() * 100000)}`)
          }
          // 处理特殊情况
          if (item.fixed || (item.title && item.title === '操作') || (item.dataIndex && item.dataIndex === 'action')) {
            item.disabled = true
          }
        })
        const lastColumn = columns[columns.length - 1]
        if (!lastColumn.hasOwnProperty('scopedSlots')) {
          this.$set(lastColumn, 'scopedSlots', {})
        }
        this.$set(lastColumn.scopedSlots, 'filterDropdown', 'setConfigDropdown')
        this.$set(lastColumn.scopedSlots, 'filterIcon', 'setConfigIcon')
      }
      this.setCheckAll()
    },
    /**
     * 设置全选按钮 状态
     */
    setCheckAll () {
      const checked = this.localSetConfig.columns.find(item => {
        // 只要有一个false 则全选按钮为false
        return item.checked === false
      })
      this.checkboxAllChecked = !checked
      this.indeterminate = !!checked
      this.changeColumns()
    },
    /**
     * 全选
     * @param {Event} event
     */
    handleCheckAllChange (event) {
      this.localSetConfig.columns.forEach(item => {
        // 屏蔽状态的不做处理
        if (!item.disabled) {
          item.checked = event.target.checked
        }
      })
      this.setCheckAll()
    },
    /**
     * 单选
     * @param {Event} event
     * @param {Number} index
     */
    handleCheckChange (event, index) {
      const item = this.localSetConfig.columns[index]
      if (item) {
        item.checked = event.target.checked
        this.setCheckAll()
      }
    },
    /**
     * 拖拽
     * @param {Object} e
     * @param {Event} originalEvent
     * @returns {Boolean}
     */
    handleDraggableMove (e, originalEvent) {
      if (e.relatedContext.element.fixed) {
        this.$message.info({ content: '禁止放在固定列之后', key: 'updatable' })
        return false
      }
      return true
    },
    /**
     * 拖拽完成
     * @param {Object} e
     * @param {Event} originalEvent
     * @returns {Boolean}
      */
    handleDraggableEnd () {
      this.changeColumns()
    },
    /**
     * 同步columns
     */
    changeColumns () {
      this.$emit('changeColumns', this.localSetConfig.columns)
    }
  },
  render () {
    const props = { ...this.$attrs }
    let columns = []
    if (this.setConfig) {
      columns = this.localSetConfig.columns.filter(item => {
        return item.checked
      })
      props.columns = columns
    }
    const title = <span class="title">请选择表头中要展示的信息</span>
    const checkboxAllProps = {
      props: {
        indeterminate: this.indeterminate,
        checked: this.checkboxAllChecked
      },
      class: 'check-all-container',
      on: {
        change: this.handleCheckAllChange
      }
    }
    const checkboxAll = <a-checkbox {...checkboxAllProps} >全部</a-checkbox>
    const br = <br />
    const setconfigContainerProps = {
      class: 'setconfig-container'
    }
    const setconfigContainer =
       <div {...setconfigContainerProps}>
         {title}
         {checkboxAll}
         {br}
         <div class='check-children-container'>
           <draggable list={this.localSetConfig.columns} handle='.check-item-container' filter=".check-item-disabled-container" onEnd={this.handleDraggableEnd} move={this.handleDraggableMove }>
             {this.localSetConfig.columns.map((item, index) => {
               let title = ''
               if (item.hasOwnProperty('title')) {
                 title = item.title
               } else if (item.slots && item.slots.title) {
                 let vnode = this.$slots[item.slots.title]
                 if (vnode && Object.prototype.toString.call(vnode) === '[object Array]') {
                   vnode = vnode[0]
                   title = vnode.children && Object.prototype.toString.call(vnode.children) === '[object Array]' ? vnode.children[0].text : ''
                 }
               }
               return (<div class={item.disabled ? 'check-item-disabled-container' : 'check-item-container'} >

                 <a-checkbox class='check-item' disabled={item.disabled} checked={item.checked} defaultChecked={item.checked} onChange={(event) => this.handleCheckChange(event, index)}>{title}</a-checkbox>
               </div>)
             })}
           </draggable>
         </div>
       </div>

    const setConfigIconProps = {
      props: {
        type: 'unordered-list'
      },
      class: 'setconfig-icon'
    }
    const setConfigIcon = <a-icon {...setConfigIconProps}></a-icon>
    const scopedSlots = { ...this.$scopedSlots, setConfigDropdown: props => setconfigContainer, setConfigIcon: props => setConfigIcon }

    return (
      <div>
        <a-table class="fw-affixtable" props={props} on={this.$listeners} scopedSlots={ scopedSlots }>
          {Object.keys(this.$slots).map(name => (<template slot={name}>{this.$slots[name]}</template>))}

        </a-table>
      </div>
    )
  }
}
